program picprog;                                        {Program INHX into PIC}	{pi030519}
uses errmsg60,picunit;                                  {Uses errmsg60, picunit}
var
  f: text;                                              {Text file}
  sum: byte;                                            {Checksum}

type
  hilo =                                                {Convert word/byte}
  record
    lo,hi: byte;
  end;

function rdhex: byte;                                   {Writes hex to file}
var
  i: word;
  x: byte;
  ch: char;
begin
  x:=0;
  for i:=0 to 1 do
  begin
    read(f,ch);
    case ch of
      '0'..'9': x := x shl 4 + ord(ch) - 48;
      'A'..'F': x := x shl 4 + ord(ch) - 55;
      'a'..'f': x := x shl 4 + ord(ch) - 87;
    else
      writeln('Error in HEX file'); halt;
    end;
  end;
  rdhex:=x;
  sum:=sum+x;
end;

procedure writebin( x : word; digits : integer );
  var i :integer;
  begin
    for i := digits-1 downto 0 do begin
      if x and (1 shl i) = 0 then
        write('0')
      else
        write('1');
    end; {endfor}
  end {writebin};

var
  pgmmembyte: array[0..2*PgmMemSize-1] of byte absolute pgmmem;          	{pi990826}
  cfgmembyte: array[$4000..$4000+2*CfgMemSize-1] of byte absolute cfgmem;      	{pi990826}
  datmembyte: array[$4200..$4200+2*DatMemSize-1] of byte absolute datmem;      	{pi990826}
  pgmm: pgmmemtype;
  datm: datmemtype;
  cfgm: cfgmemtype;
  ch: char;
  ant: byte;
  x: byte;
  adr: word;
  i: integer;
  ok: boolean;
  err:byte;
  fn: string;
  prog_flag, data_flag, cfg_flag: boolean;  					{pi980306}
  eraseall_flag : boolean;					                {pi991013}
  device : integer; {microchip device id}                                       {pi991019}
  size_pgm_mem : integer;                                                       {pi991019}
  size_dat_mem : integer;
  used_pgm_mem : integer;
  used_dat_mem : integer;
  delay_time : word;

begin
  fillchar(pgmmem,sizeof(pgmmem),255);                          {Clear buffer}
  fillchar(datmem,sizeof(datmem),255);
  fillchar(cfgmem,sizeof(cfgmem),255);
  prog_flag := false; data_flag := false; cfg_flag := false;                    {pi980306}
  eraseall_flag := false;                                                       {pi991013}
  size_pgm_mem := $400;  { 16F84 & 16C84 }                                      {pi991019}
  size_dat_mem := $40;   { 16F84 & 16C84 }
  delay_time := 10;
  if paramcount>=2 then val(paramstr(2),com,i) else                             {pi980306}
  begin
    writeln('Use:');
    writeln('   picprog filename.hex comport [-e] [-p] [-d] [-c]'); 	        {pi991013}
    writeln('');
    writeln('     with:     -e   do an "erase all" before programming');	{pi991013}
    writeln('               -p   program "program memory" only');		{pi991013}
    writeln('               -d   program "EE data memory" only');		{pi991013}
    writeln('               -c   program "config memory" only');		{pi991013}
    writeln('');								{pi991013}
    writeln('maxsize: pgm=',PgmMemSize,' dat=',DatMemSize,' cfg=',CfgMemSize ); {pi990826}
    writeln('Version 20030519 for 16C84, 16F84, 16F84A and 16F87X');
    halt;
  end;

  for i := 3 to paramcount do begin                             		{pi980306}
     if paramstr(i)='-p' then prog_flag := true;
     if paramstr(i)='-P' then prog_flag := true;
     if paramstr(i)='/p' then prog_flag := true;
     if paramstr(i)='/P' then prog_flag := true;
     if paramstr(i)='-d' then data_flag := true;
     if paramstr(i)='-D' then data_flag := true;
     if paramstr(i)='/d' then data_flag := true;
     if paramstr(i)='/D' then data_flag := true;
     if paramstr(i)='-c' then cfg_flag := true;
     if paramstr(i)='-C' then cfg_flag := true;
     if paramstr(i)='/c' then cfg_flag := true;
     if paramstr(i)='/C' then cfg_flag := true;
     if paramstr(i)='-e' then eraseall_flag := true;				{pi991013}
     if paramstr(i)='-E' then eraseall_flag := true;				{pi991013}
     if paramstr(i)='/e' then eraseall_flag := true;				{pi991013}
     if paramstr(i)='/E' then eraseall_flag := true;				{pi991013}
  end;

  { if no flags are found assume "all" (compatible to older version) } 		{pi980306}
  if not (prog_flag or data_flag or cfg_flag)               			{pi991019}
  then
  begin
     prog_flag := true; data_flag := true; cfg_flag := true;
  end;

  fn:=paramstr(1);
  filename:=@fn;

  assign(f,fn);                                                 {Open file}
  reset(f);
  writeln('Reading file ''',paramstr(1),'''');

  used_pgm_mem := 0;
  used_dat_mem := 0;
  prog_cycle_delay := 10;

  repeat                                                        {Read hex file}
    read(f,ch);
    if not eof(f) and (ch=':') then
    begin
      sum:=0;
      ant:=rdhex;
      hilo(adr).hi:=rdhex;
      hilo(adr).lo:=rdhex;
      case rdhex of
        0:
          begin
            for i:=1 to ant do
              begin
                case adr of
                  $0000..$0000+2*PgmMemSize-1:
                    begin
                      pgmmembyte[adr]:=rdhex; {pi990826}
                    end;
                  $4000..$4000+2*CfgMemSize-1:
                    begin
                      cfgmembyte[adr]:=rdhex; {pi990826}
                    end;
                  $4200..$4200+2*DatMemSize-1:
                    begin
                      datmembyte[adr]:=rdhex; {pi990826}
                    end;
                end {case adr of};
                inc(adr);
              end {for};
            case adr of
              $0000..$0000+2*PgmMemSize-1:
                if adr>used_pgm_mem then used_pgm_mem := adr;
              $4200..$4200+2*DatMemSize-1:
                if adr-$4200>used_dat_mem then used_dat_mem := adr-$4200
              else
                ;
            end {case adr of};
          end {case 0};
        1:
          begin
            i:=0;
          end;
      end {case rdhex of};
      x:=rdhex;
      if sum<>0 then
      begin
        writeln('Checksum error'); halt;
      end;
    end;
    readln(f);
    if eof(f) then i:=0;
  until i=0;
  close(f);                                                     {Close file}

  filename:=NIL;

  used_pgm_mem := (used_pgm_mem+1) div 2;
  used_dat_mem := (used_dat_mem+1) div 2;

  writeln( used_pgm_mem, ' program locations and ',
           used_dat_mem, ' (EE) data locations read' );

  if used_pgm_mem>size_pgm_mem then
    begin
      writeln( '*** Error: to much code ***');
      used_pgm_mem := size_pgm_mem;
    end;

  if used_dat_mem>size_dat_mem then
    begin
      writeln( '*** Error: to much (EE) data ***');
      used_dat_mem := size_dat_mem;
    end;

  if cfgmem[7]=$ffff then
    writeln('--- Notice: No configuration word in hex-file ---');

  pgmm:=pgmmem;                                                 {Verify copy}
  datm:=datmem;
  cfgm:=cfgmem;

  writeln('PIC PROGRAMMER at COM',com);

  err:=hwerror;                                 {HW-check}
  case err of
    1: writeln ('- Comport not exist');
    2: writeln ('- Modem found');
    3: writeln ('- Ilegal value of parameter');
    4: writeln ('- Programmer not found');
    5: writeln ('- Cable too long');
    6: writeln ('- Timer 2 not free');
  end;
  if err<>0 then halt;

  readcfg;									{pi991013}
  write( 'Device ID is ' );							{pi991013}
  writebin( cfgmem[6], 14 );
  write( 'b (Dev=' );								{pi991013}
  device := cfgmem[6] shr 5;                    				{pi991019}
  case device of                                				{pi991019}
    PIC16F84A:
      begin
        write( 'PIC16F84A' );
        size_pgm_mem := $0400;  { 1K }
        size_dat_mem := $0040;  { 64 }
        delay_time := 10;
      end;
    PIC16F877:
      begin
        write( 'PIC16F877' );
        size_pgm_mem := $2000;  { 8K  }
        size_dat_mem := $0100;  { 256 }
        delay_time := 2;
      end;
    PIC16F876:
      begin
        write( 'PIC16F876' );  {??}
        size_pgm_mem := $2000;  { 8K }
        size_dat_mem := $0100;  { 256 }
        delay_time := 2;
      end;
    PIC16F874:
      begin
        write( 'PIC16F874' );  {??}
        size_pgm_mem := $1000;  {4K}
        size_dat_mem := $0080;
        delay_time := 2;
      end;
    PIC16F873:
      begin
        write( 'PIC16F873' );  {??}						{pi991013}
        size_pgm_mem := $1000;  {4K}     					{pi991019}
        size_dat_mem := $0080;
        delay_time := 2;
      end
  else										{pi991013}
     write( 'unknown' );							{pi991013}
     size_pgm_mem := $0400;  {1K: 16F84}         				{pi991019}
     size_dat_mem := $0040;
     delay_time := 10;
  end; {endcase};								{pi991013}
  writeln( ', Rev=', cfgmem[6] and $1F, ')');					{pi991013}

  if eraseall_flag then 							{pi991013}
    begin									{pi991013}
      writeln( 'Erasing All' );							{pi991013}
      eraseall;									{pi991013}
    end 									{pi991013}
  else {if eraseall_flag}							{pi991013}
    begin									{pi991013}
      { readcfg;								 pi991013}
      if cfgmem[7] and $30 <> $30 then						{pi991013}
        begin
          write('- Code Protected. Erase (Y/N)? ');
          repeat
            readln(ch);
            if upcase(ch)='N' then halt;
          until upcase(ch)='Y';
          writeln( 'Erasing All' );
          eraseall;
        end;
    end {if eraseall_flag};							{pi991013}

  prog_cycle_delay := delay_time;

  { setting below 5 does not work. bug in waitdelay ??? }
  if prog_cycle_delay<5 then prog_cycle_delay := 5;

  if used_pgm_mem <= 0 then prog_flag := false;
  if used_dat_mem <= 0 then data_flag := false;

                                                                {Program}
  if prog_flag then                                             		{pi980306}
  begin
     writeln( 'Programming Program Memory' );
     progcod( used_pgm_mem );
  end;

  if data_flag then                                             		{pi980306}
  begin
     writeln( 'Programming EE Data Memory' );
     progdat( used_dat_mem );
  end;

  if prog_flag then
  begin
    write('Verify Program Memory ');                            {Verify}
    readcod( used_pgm_mem );     						{pi980306}
    ok:=true;
    for i:=0 to used_pgm_mem-1 do 
       if pgmmem[i]<>(pgmm[i] and $3fff) then ok:=false;			{pi990826}
    writeln(ok);
  end;

  if data_flag then
  begin
    write('Verify EE Data Memory ');              	        {Verify}
    readdat( used_dat_mem );                                    		{[pi980306]}
    ok:=true;
    for i:=0 to used_dat_mem-1 do
      if lo(datmem[i])<>lo(datm[i]) then
        ok:=false;
    writeln(ok);
  end;

  if cfg_flag then                                              		{pi980306}
  begin
    writeln('Programming configuration word');
    cfgmem:=cfgm;
    progcfg;
    readcfg;  { reads into "cfgmem" }
    if cfgmem[7] and $30 <> $30 then						{pi991013}
      writeln('Device code-protected')
    else
      begin
        write('Verify ');
        for i:=0 to 3 do if cfgmem[i]<>(cfgm[i] and $3fff) then
          begin
            ok:=false;
          end;
        if cfgmem[7]<>(cfgm[7] and $3fff) then
          begin
            ok:=false;
          end;
        writeln(ok);
        if cfgmem[7]<>(cfgm[7] and $3fff) then
          begin
            write( '*** Error in configuration: the value is ' );
            writebin( cfgmem[7],14);
            write( 'b but should be ' );
            writebin( cfgm[7],14 );
            writeln( 'b' );
          end;
      end {else};
  end {if cfg_flag};
end.
