program picread;                                        {Read into inhx file} {pi19992010}
uses errmsg60,picunit;                                  {Uses picunit}
var
  f: text;                                              {Text file}
  sum: byte;                                            {Checksum}

type
  hilo =                                                {Convert byte/words}
  record
    lo,hi: byte;
  end;

procedure wrhex(data: byte);                            {Writes hex to file}
const
  hx: array[0..15] of char =                            {Define hex chars}
    ('0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F');
begin
  sum:=sum+data;                                        {Update checksum}
  write(f,hx[data shr 4],hx[data and $f]);              {Write hex}
end;

procedure outhex(var memory; adr: word; count: byte);   {Output hex line}
var
  mem: array[1..255] of byte absolute memory;           {Memory}
  i: word;                                              {Count variable}
begin
  sum:=0;                                               {Reset checksum}
  write(f,':');                                         {Begin line}
  wrhex(count);                                         {Number of data}
  wrhex(hi(adr));                                       {High adress}
  wrhex(lo(adr));                                       {Low adress}
  wrhex(0);                                             {0 = data record}
  for i:=1 to count do wrhex(mem[i]);                   {Write data}
  wrhex(-sum);                                          {Checksum}
  writeln(f);                                           {Next line}
end;

procedure writebin( x : word; digits : integer );
  var i :integer;
  begin
    for i := digits-1 downto 0 do begin
      if x and (1 shl i) = 0 then
        write('0')
      else
        write('1');
    end; {endfor}
  end {writebin};

var
  i: word;                                                       {Count}
  err: byte;
  fn: string;
  prog_flag, data_flag, cfg_flag: boolean;                                    {pi19992010}
  device : integer; {microchip device id}                                     {pi19992010}
  size_pgm_mem : integer;                                                      {pi19992010}
  size_dat_mem : integer;

begin
  fillchar(pgmmem,sizeof(pgmmem),255);                          {Clear buffer}
  fillchar(datmem,sizeof(datmem),255);
  fillchar(cfgmem,sizeof(cfgmem),255);
  prog_flag := false; data_flag := false; cfg_flag := false;                  {pi19992010}
  if paramcount>=2 then val(paramstr(2),com,i) else              {Get comport}{pi19992010}
  begin                                                         {Else help}
    writeln('Use:');
    writeln('   picread filename.hex comport [-p] [-d] [-c]'); 	 	      {pi19992010}
    writeln('');
    writeln('     with:     -p   read "program memory" only');		      {pi19992010}
    writeln('               -d   read "EE data memory" only');		      {pi19992010}
    writeln('               -c   read "config memory" only');		      {pi19992010}
    writeln('');					                      {pi19992010}
    writeln('Version 20030519 for 16C84, 16F84, 16F84A and 16F87X');
    halt;
  end;

  for i := 3 to paramcount do begin                             	      {pi19992010}
     if paramstr(i)='-p' then prog_flag := true;
     if paramstr(i)='-P' then prog_flag := true;
     if paramstr(i)='/p' then prog_flag := true;
     if paramstr(i)='/P' then prog_flag := true;
     if paramstr(i)='-d' then data_flag := true;
     if paramstr(i)='-D' then data_flag := true;
     if paramstr(i)='/d' then data_flag := true;
     if paramstr(i)='/D' then data_flag := true;
     if paramstr(i)='-c' then cfg_flag := true;
     if paramstr(i)='-C' then cfg_flag := true;
     if paramstr(i)='/c' then cfg_flag := true;
     if paramstr(i)='/C' then cfg_flag := true;
  end;

  { if no flags are found assume "all" (compatible to older version) } {[pi980306]}
  if not (prog_flag or data_flag or cfg_flag)               	{pi991019}
  then
  begin
     prog_flag := true; data_flag := true; cfg_flag := true;
  end;

  fn:=paramstr(1);
  writeln('Reading Processor at COM',com,' to ''',fn,'''');      {Info}

  err:=hwerror;                                 {HW-check}
  case err of
    1: writeln ('- Comport not exist');
    2: writeln ('- Modem found');
    3: writeln ('- Ilegal value of parameter');
    4: writeln ('- Programmer not found');
    5: writeln ('- Cable too long');
    6: writeln ('- Timer 2 not free');
  end;
  if err<>0 then halt;

  writeln( 'Reading Configuration Memory' );
  readcfg;                                                      {Read config}
  write( 'Device ID is ' );			{pi991013}
  writebin( cfgmem[6], 14 );
  write( 'b (Dev=' );				{pi991013}
  device := cfgmem[6] shr 5;                    {pi991019}
  case device of                                {pi991019}
    PIC16F84A:
      begin
        write( 'PIC16F84A' );
        size_pgm_mem := $0400;  { 1K }
        size_dat_mem := $0040;  { 64 }
        {delay_time := 10;}
      end;
    PIC16F877:
      begin
        write( 'PIC16F877' );
        size_pgm_mem := $2000;  { 8K  }
        size_dat_mem := $0100;  { 256 }
        {delay_time := 2;}
      end;
    PIC16F876:
      begin
        write( 'PIC16F876' );  {??}
        size_pgm_mem := $2000;  { 8K }
        size_dat_mem := $0100;  { 256 }
        {delay_time := 2;}
      end;
    PIC16F874:
      begin
        write( 'PIC16F874' );  {??}
        size_pgm_mem := $1000;  {4K}
        size_dat_mem := $0080;
        {delay_time := 2;}
      end;
    PIC16F873:
      begin
        write( 'PIC16F873' );  {??}	{pi991013}
        size_pgm_mem := $1000;  {4K}     {pi991019}
        size_dat_mem := $0080;
        {delay_time := 2;}
      end
  else						{pi991013}
     write( 'unknown' );			{pi991013}
     size_pgm_mem := $0400;  {1K: 16F84}         {pi991019}
     size_dat_mem := $0040;
     {delay_time := 10;}
  end; {endcase};				{pi991013}
  writeln( ', Rev=', cfgmem[6] and $1F, ')');	{pi991013}

  if cfgmem[7] and $30 <> $30 then		{pi991013}
    begin
      write('- Code Protected! ');
      halt;
    end;

  filename:=@fn;                                                {Set filename}
  assign(f,fn);                                                 {Open file}
  rewrite(f);                                                   {Write mode}

  if prog_flag then                                             {[pi980306]}
  begin
     writeln( 'Reading Program Memory' );
     readcod( size_pgm_mem );
     writeln( 'storing Program Memory' );
     for i:=0 to (2*size_pgm_mem div $10)-1 do                               {Write pgmmem}
       outhex(pgmmem[i*8],i*$10,$10);
  end;

  if cfg_flag then
  begin
     writeln( 'storing Configuration Memory' );
     outhex(cfgmem[0],$4000,8); { 0..3 }                                {Write id's}
     outhex(cfgmem[4],$4008,4); { 4..5 }                                {resverved}
     outhex(cfgmem[6],$400C,2); { 6 }  					{DEVICE ID}
     outhex(cfgmem[7],$400E,2);	{ 7 }	                                {Write cfgword}
  end; {endif}

  if data_flag then                                             {[pi980306]}
  begin
     writeln( 'Reading EE Data Memory' );
     readdat( size_dat_mem );
     writeln( 'storing EE Data Memory' );
     for i:=0 to size_dat_mem-1 do
        hilo(datmem[i]).hi:=0;                     {Clr high data}
     for i:=0 to (2*size_dat_mem div $10)-1 do
        outhex(datmem[i*8],i*$10+$4200,$10);         {Write datmem}
  end;

  writeln(f,':00000001FF');                                     {End data}
  close(f);                                                     {Close file}

end.
