program pictest;                        {TEST PIC16c84}
uses crt,picunit;                       {Uses picunit}

const
  tc = longint(55);                     {Delay between ticks}

var
  pgmmem1: pgmmemtype;                  {Program memory, copy}
  datmem1: datmemtype;                  {Data memory, copy}
  cfgmem1: cfgmemtype;                  {Config memory, copy}
  i: word;                              {Counter etc}
  ok: boolean;                          {OK flag}
  t1: longint;                          {Store time}
  t: longint absolute 0:$46c;           {Timer}
  err: byte;
  size_pgm_mem : integer;                                                       {pi030519}
  size_dat_mem : integer;							{pi030519}

begin
  size_pgm_mem := $0400;  { 1K }						{pi030519}
  size_dat_mem := $0040;  { 64 }						{pi030519}
  if paramcount=1 then val(paramstr(1),com,i) else
  begin
    writeln('Use:');
    writeln('   pictest comport');
    halt;
  end;

  writeln('Testing PIC-Programmer at COM',com);

  err:=hwerror;                                 {HW-check}
  case err of
    1: writeln ('- Comport not exist');
    2: writeln ('- Modem found');
    3: writeln ('- Ilegal value of parameter');
    4: writeln ('- Programmer not found');
    5: writeln ('- Cable too long');
    6: writeln ('- Timer 2 not free');
  end;
  if err<>0 then halt;

  writeln('- Erase');
  eraseall;

  write('- Blank check ');
  t1:=t;
  readall( size_pgm_mem, size_dat_mem );					{pi030519}
  ok:=true;
  for i:=0 to $3ff do if pgmmem[i]<>$3fff then ok:=false;
  t1:=t-t1;
  writeln(ok,', ',t1*tc,'ms');

  writeln('- Generate random numbers');
  randomize;
  for i:=0 to $3ff do pgmmem[i]:=random($3fff);
  for i:=0 to $3f  do datmem[i]:=random($ff);
  pgmmem1:=pgmmem; datmem1:=datmem;

  write('- Programming random numbers, ');
  t1:=t;
  progall( size_pgm_mem, size_dat_mem );					{pi030519}
  t1:=t-t1;
  writeln(t1*tc,'ms');

  write('- Verify ');
  t1:=t;
  readall( size_pgm_mem, size_dat_mem );					{pi030519}
  ok:=true;
  for i:=0 to $3ff do if pgmmem[i]<>pgmmem1[i] then ok:=false;
  for i:=0 to $3f do if lo(datmem[i])<>lo(datmem1[i]) then ok:=false;
  t1:=t-t1;
  writeln(ok,', ',t1*tc,'ms');

  write('- Programming ZERO''s, ');
  t1:=t;
  fillchar(pgmmem,sizeof(pgmmem),0);
  fillchar(datmem,sizeof(datmem),0);
  progall( size_pgm_mem, size_dat_mem );					{pi030519}
  t1:=t-t1;
  writeln(t1*tc,'ms');

  write('- Verify ');
  t1:=t;
  readall( size_pgm_mem, size_dat_mem );					{pi030519}
  ok:=true;
  for i:=0 to $3ff do if pgmmem[i]<>0 then ok:=false;
  for i:=0 to $3f do if lo(datmem[i])<>0 then ok:=false;
  t1:=t-t1;
  writeln(ok,', ',t1*tc,'ms');
end.
