unit picunit;                           { PIC16C84-programmer unit, v27 } 	{pi030519}
interface                               { By Jens Dyekjr Madsen        }

const
  PgmMemSize = $2000;                   					{pi991019}
  DatMemSize = $100;                    					{pi991019}
  CfgMemSize = $8;                      					{pi990826}
  MaxMemSize = $2000; { = MAX( PgmMemSize, DatMemSize, CfgMemSize )     } 	{pi990826}

  PIC16F84A = 43;  { Microchip Device Ids [bit5..bit13] @ 2006h }         	{pi030519}
  PIC16F870 = 104;
  PIC16F871 = 105;
  PIC16F872 = 71;
  PIC16F873 = 75;
  PIC16F874 = 73;
  PIC16F876 = 79;
  PIC16F877 = 77;

type
  pgmmemtype = array[0..PgmMemSize-1] of word;  { Program memory type   } 	{pi990826}
  datmemtype = array[0..DatMemSize-1] of word;  { Data memory type      } 	{pi990826}
  cfgmemtype = array[0..CfgMemSize-1] of word;  { Config memory type    } 	{pi990826}

var
  com: byte;                            { COM port                      }
  pgmmem: pgmmemtype;                   { Program memory                }
  datmem: datmemtype;                   { Data memory                   }
  cfgmem: cfgmemtype;                   { Config memory                 }
  prog_cycle_delay : word;

procedure eraseall;                     { Erase code                    }
procedure readcfg;                      { Read config                   }
procedure readall( codSz,datSz:integer);{ Read code+data                }
procedure readcod( codSz : integer );   { Read code        [pi980306]   }
procedure readdat( datSz : integer );   { Read data        [pi980306]   }
procedure progcfg;                      { Program config                }
procedure progall( codSz,datSz:integer);{ Program code+data             }
procedure progcod( codSz : integer );   { Program code     [pi980306]   }
procedure progdat( datSz : integer );   { Program data     [pi980306]   }
function hwerror: byte;                 { Check and return error code<>0}


implementation
uses delays,jdm84io;                    { Use delay and I/O             }

const                                   { Define Commands               }
  LoadCfg  = $1600; {00 0000}           { Load Configuration            }
  LoadPgm  = $1602; {00 0010}           { Load Data for Program Memory  }
  ReadPgm  = $1604; {00 0100}           { Read Data from Program Memory }
  IncAdr   = $0606; {00 0110}           { Increment Address             }
  BegPgm   = $0608; {00 1000}           { Begin Programming             }
  LoadData = $1603; {00 0011}           { Load Data for Data Memory     }
  ReadData = $1605; {00 0101}           { Read Data from Data Memory    }
  ErasePgm = $0609; {00 1001}           { Bulk Erase Program Memory     }
  EraseData= $060b; {00 1011}           { Bulk Erase Data Memory        }
  EraseAll1= $0601; {00 0001}           { EraseAll, first               }
  EraseAll2= $0607; {00 0111}           { EraseAll, next                }

var
  data: integer;                        { Return data                   }

procedure start;                        { Start                         }
begin
  setcom(com);                          { Setup com-port                }
  if err=0 then mclr;                   { Reset if ok                   }
end;

procedure stop;                         { Stop                          }
begin
  mclr;                                 { Reset / go                    }
end;

procedure pgm(cmd, d: integer);         { Read/write commands serial    }
var
  c: longint;                           { Shift variable                }
  i: integer;                           { Count variable                }
begin
  c:=lo(cmd)+longint(d) shl 7;          { Setup bits                    }
  data:=0;                              { Clear input bits              }
  begincmd;                             { Begin command (cli,init timer)}
  for i:=1 to hi(cmd) do                { Shift bits                    }
  begin
    data:=data shr 1 + din shl 13;      { Read data                     }
    dout(c and 1);                      { Write data                    }
    c:=c shr 1;                         { Next bit                      }
  end;
  endcmd;                               { End command (close timer, sti)}
end;

function hwerror: byte;                 { Check if programmer ok        }
begin
  start;                                { Start                         }
  hwerror:=err;                         { Error if not ok               }
  stop;                                 { Stop                          }
end;

procedure rdmem(VAR memory; cmd,count:integer); { Read data to memory   }
var
  mem: array[0..MaxMemSize-1] of word absolute memory;  { Memory        } 	{pi990826}
  adr: integer;                                 { Adress counter        }
begin
  for adr:=0 to count do                        { Adress 0..count       }
  begin
    pgm(cmd,-1);                                { Read                  }
    mem[adr]:=data;                             { Store data            }
    pgm(incadr,0);                              { Next adress           }
  end;
end;

procedure progmem(VAR memory; rdcmd, ldcmd, count:integer);
var
  mem: array[0..MaxMemSize-1] of word absolute memory;   		  	{pi990826}
  adr: integer;                                 { Adress counter        }
begin
  for adr:=0 to count do                        { Adress 0..count       }
  begin
    pgm(rdcmd,-1);                              { Read                  }
    if data<>mem[adr] then                      { Need to be programmed?}
    begin
      pgm(ldcmd,mem[adr]);                      { Load                  }
      pgm(BegPgm,0);                            { Begin programming     }
      waitdelay( prog_cycle_delay );            { Delay 10ms            }
    end;
    pgm(incadr,0);                              { Next adress           }
  end;
end;

procedure eraseall;                             { Erase code            }
var
  i: integer;                                   { Adress counter        }
begin
  start;                                        { Start, Reset          }
  pgm(loadcfg,$3fff);                           { Setup config word     }
  for i:=1 to 7 do pgm(incadr,0);               { Adress at config word }
  pgm(Loadpgm,$3fff);                           { Load config word      }
  pgm(EraseAll1,0);                             { Start erase           }
  pgm(EraseAll2,0);
  pgm(BegPgm,0);                                { Start programming     }
  waitdelay(20);                                { Delay 20ms            }
  pgm(EraseAll1,0);                             { Stop erase            }
  pgm(EraseAll2,0);
  stop;                                         { Stop                  }
end;

procedure readcfg;                              { Read config           }
begin
  start;                                        { Start, Reset          }
  pgm(loadcfg,0);                               { Config area           }
  rdmem(cfgmem,readpgm,CfgMemSize-1);           { Read config           } 	{pi990826}
  stop;                                         { Stop                  }
end;

procedure readall( codSz, datSz : integer);     { Read code+data        }
begin
  start;                                        { Start, Reset          }
  rdmem(pgmmem,readpgm,codSz-1);                { Read program          } 	{pi990826}
  rdmem(datmem,readdata,datSz-1);               { Read data             } 	{pi990826}
  stop;                                         { Stop                  }
end;

procedure readcod( codSz : integer );           { Read code             }
begin
  start;                                        { Start, Reset          }
  rdmem(pgmmem,readpgm,codSz-1);                { Read program          } 	{pi990826}
  stop;                                         { Stop                  }
end;

procedure readdat( datSz : integer );           { Read data             }
begin
  start;                                        { Start, Reset          }
  rdmem(datmem,readdata,datSz-1);               { Read data             } 	{pi990826}
  stop;                                         { Stop                  }
end;

procedure progcfg;                              { Program config        }
begin
  start;                                        { Start, Reset          }
  pgm(loadcfg,0);                               { Config area           }
  progmem(cfgmem,readpgm,loadpgm,CfgMemSize-1); { PGM config            } 	{pi990826}
  stop;                                         { Stop                  }
end;

procedure progall( codSz,datSz:integer);        { Program code+data     }
begin
  start;                                        { Start, Reset          }
  progmem(pgmmem,readpgm,loadpgm,codSz-1);      { PGM program           } 	{pi990826}
  progmem(datmem,readdata,loaddata,datSz-1);    { PGM data              } 	{pi990826}
  stop;                                         { Stop                  }
end;

procedure progcod( codSz : integer );           { Program code          }
begin
  start;                                        { Start, Reset          }
  progmem(pgmmem,readpgm,loadpgm,codSz-1);      { PGM program           } 	{pi990826}
  stop;                                         { Stop                  }
end;

procedure progdat( datSz : integer );           { Program data          }
begin
  start;                                        { Start, Reset          }
  progmem(datmem,readdata,loaddata,datSz-1);    { PGM data              } 	{pi990826}
  stop;                                         { Stop                  }
end;

begin
  prog_cycle_delay := 10;
end.